@@ -2,6 +2,7 @@ from __future__ import absolute_import
 from __future__ import print_function
 from __future__ import unicode_literals
 
+import base64
 import collections
 import sys
 from importlib import import_module
@@ -19,9 +20,9 @@ from .helpers import create_superuser
 from .helpers import DUMMY_PASSWORD
 from .helpers import provision_device
 from kolibri.core import error_constants
+from kolibri.core.auth.backends import FACILITY_CREDENTIAL_KEY
 from kolibri.core.device.utils import set_device_settings
 
-
 # A weird hack because of http://bugs.python.org/issue17866
 if sys.version_info >= (3,):
 
@@ -163,6 +164,18 @@ class LearnerGroupAPITestCase(APITestCase):
         self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
         self.assertEqual(response.data[0]["id"], error_constants.UNIQUE)
 
+    def test_cannot_create_learnergroup_no_classroom_parent(self):
+        classroom_id = self.classrooms[0].id
+        learner_group_id = (
+            models.LearnerGroup.objects.filter(parent_id=classroom_id).first().id
+        )
+        response = self.client.post(
+            reverse("kolibri:core:learnergroup-list"),
+            {"parent": learner_group_id, "name": "some name"},
+            format="json",
+        )
+        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
+
 
 class ClassroomAPITestCase(APITestCase):
     @classmethod
@@ -236,7 +249,15 @@ class ClassroomAPITestCase(APITestCase):
                     "is_superuser": True,
                     "full_name": self.superuser.full_name,
                     "username": self.superuser.username,
-                    "roles": [],
+                    "roles": [
+                        {
+                            "collection": self.facility.id,
+                            "kind": role_kinds.ASSIGNABLE_COACH,
+                            "id": self.superuser.roles.get(
+                                collection=self.facility.id
+                            ).id,
+                        }
+                    ],
                 }
             ],
         }
@@ -276,6 +297,17 @@ class ClassroomAPITestCase(APITestCase):
         }
         self.assertDictEqual(response.data, expected)
 
+    def test_classroom_facility_coach_role_for_filter(self):
+        coach = FacilityUserFactory.create(facility=self.facility)
+        self.facility.add_coach(coach)
+        response = self.client.get(
+            reverse("kolibri:core:classroom-list"),
+            data={"role": "coach"},
+            format="json",
+        )
+        # Should return all classrooms
+        self.assertEqual(len(response.data), len(self.classrooms))
+
     def test_cannot_create_classroom_same_name(self):
         classroom_name = self.classrooms[0].name
         response = self.client.post(
@@ -286,6 +318,15 @@ class ClassroomAPITestCase(APITestCase):
         self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
         self.assertEqual(response.data[0]["id"], error_constants.UNIQUE)
 
+    def test_cannot_create_classroom_no_facility_parent(self):
+        classroom_id = self.classrooms[0].id
+        response = self.client.post(
+            reverse("kolibri:core:classroom-list"),
+            {"parent": classroom_id, "name": "another name"},
+            format="json",
+        )
+        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
+
 
 class FacilityAPITestCase(APITestCase):
     @classmethod
@@ -418,6 +459,64 @@ class FacilityAPITestCase(APITestCase):
         response = self.client.get(reverse("kolibri:core:publicfacility-list"))
         self.assertEqual(models.Facility.objects.all().count(), len(response.data))
 
+    def test_public_facilityuser_endpoint(self):
+        if sys.version_info[0] == 2:
+            credentials = base64.b64encode(
+                "username={}&{}={}:{}".format(
+                    self.user1.username,
+                    FACILITY_CREDENTIAL_KEY,
+                    self.facility1.id,
+                    DUMMY_PASSWORD,
+                ).encode("utf-8")
+            )
+        else:
+            credentials = base64.b64encode(
+                str.encode(
+                    "username={}&{}={}:{}".format(
+                        self.user1.username,
+                        FACILITY_CREDENTIAL_KEY,
+                        self.facility1.id,
+                        DUMMY_PASSWORD,
+                    )
+                )
+            ).decode("ascii")
+        self.client.credentials(HTTP_AUTHORIZATION="Basic {}".format(credentials))
+        response = self.client.get(
+            reverse("kolibri:core:publicuser-list"),
+            format="json",
+        )
+        self.assertEqual(len(response.data), 1)
+        if sys.version_info[0] == 2:
+            credentials = base64.b64encode(
+                "username={}&{}={}:{}".format(
+                    self.superuser.username,
+                    FACILITY_CREDENTIAL_KEY,
+                    self.facility1.id,
+                    DUMMY_PASSWORD,
+                ).encode("utf-8")
+            )
+        else:
+            credentials = base64.b64encode(
+                str.encode(
+                    "username={}&{}={}:{}".format(
+                        self.superuser.username,
+                        FACILITY_CREDENTIAL_KEY,
+                        self.facility1.id,
+                        DUMMY_PASSWORD,
+                    )
+                )
+            ).decode("ascii")
+        self.client.credentials(HTTP_AUTHORIZATION="Basic {}".format(credentials))
+        response = self.client.get(
+            reverse("kolibri:core:publicuser-list"),
+            {"facility_id": self.facility1.id},
+            format="json",
+        )
+        self.assertEqual(
+            models.FacilityUser.objects.filter(facility_id=self.facility1.id).count(),
+            len(response.data),
+        )
+
 
 class UserCreationTestCase(APITestCase):
     @classmethod
@@ -770,14 +869,14 @@ class LoginLogoutTestCase(APITestCase):
             },
             format="json",
         )
-        response = self.client.get(
+        response = self.client.put(
             reverse("kolibri:core:session-detail", kwargs={"pk": "current"})
         )
         self.assertIn(role_kinds.ADMIN, response.data["kind"])
         self.assertIn(role_kinds.COACH, response.data["kind"])
 
     def test_session_return_anon_kind(self):
-        response = self.client.get(
+        response = self.client.put(
             reverse("kolibri:core:session-detail", kwargs={"pk": "current"})
         )
         self.assertTrue(response.data["kind"][0], "anonymous")
@@ -885,6 +984,36 @@ class AnonSignUpTestCase(APITestCase):
         self.assertEqual(response.status_code, status.HTTP_201_CREATED)
         self.assertTrue(models.FacilityUser.objects.all())
 
+    def test_no_sign_up_no_signups(self):
+        self.facility.dataset.learner_can_sign_up = False
+        self.facility.dataset.save()
+        response = self.post_to_sign_up(
+            {"username": "user", "password": DUMMY_PASSWORD}
+        )
+        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
+        self.assertFalse(models.FacilityUser.objects.all())
+        self.facility.dataset.learner_can_sign_up = True
+        self.facility.dataset.save()
+
+    def test_password_not_specified_password_required(self):
+        self.facility.dataset.learner_can_login_with_no_password = False
+        self.facility.dataset.save()
+        response = self.post_to_sign_up(
+            {"username": "user", "password": "NOT_SPECIFIED"}
+        )
+        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
+        self.assertFalse(models.FacilityUser.objects.all())
+
+    def test_password_not_specified_password_not_required(self):
+        self.facility.dataset.learner_can_login_with_no_password = True
+        self.facility.dataset.learner_can_edit_password = False
+        self.facility.dataset.save()
+        response = self.post_to_sign_up(
+            {"username": "user", "password": "NOT_SPECIFIED"}
+        )
+        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
+        self.assertTrue(models.FacilityUser.objects.all())
+
 
 class FacilityDatasetAPITestCase(APITestCase):
     @classmethod
@@ -987,6 +1116,64 @@ class FacilityDatasetAPITestCase(APITestCase):
             response = post_resetsettings()
             self.assertDictContainsSubset(mappings[setting], response.data)
 
+    def test_for_incompatible_settings_together(self):
+        self.client.login(username=self.admin.username, password=DUMMY_PASSWORD)
+        response = self.client.patch(
+            reverse(
+                "kolibri:core:facilitydataset-detail",
+                kwargs={"pk": self.facility.dataset_id},
+            ),
+            {
+                "learner_can_login_with_no_password": "true",
+                "learner_can_edit_password": "true",
+            },
+            format="json",
+        )
+        self.assertEqual(response.status_code, 400)
+
+    def test_for_incompatible_settings_sequentially(self):
+        self.client.login(username=self.admin.username, password=DUMMY_PASSWORD)
+        response = self.client.patch(
+            reverse(
+                "kolibri:core:facilitydataset-detail",
+                kwargs={"pk": self.facility.dataset_id},
+            ),
+            {
+                "learner_can_edit_password": "true",
+            },
+            format="json",
+        )
+        self.assertEqual(response.status_code, 200)
+
+        response = self.client.patch(
+            reverse(
+                "kolibri:core:facilitydataset-detail",
+                kwargs={"pk": self.facility.dataset_id},
+            ),
+            {
+                "learner_can_login_with_no_password": "true",
+            },
+            format="json",
+        )
+        self.assertEqual(response.status_code, 400)
+
+    def test_for_incompatible_settings_only_one(self):
+        # Test case handles the case when only `learner_can_login_with_no_password`
+        # is set to true in the patch request while `learner_can_edit_password`
+        # already being true due to it's default value
+        self.client.login(username=self.admin.username, password=DUMMY_PASSWORD)
+        response = self.client.patch(
+            reverse(
+                "kolibri:core:facilitydataset-detail",
+                kwargs={"pk": self.facility.dataset_id},
+            ),
+            {
+                "learner_can_login_with_no_password": "true",
+            },
+            format="json",
+        )
+        self.assertEqual(response.status_code, 400)
+
 
 class MembershipCascadeDeletion(APITestCase):
     @classmethod
@@ -1101,3 +1288,48 @@ class GroupMembership(APITestCase):
             url, {"user": self.user.id, "collection": self.classroom2.id}, format="json"
         )
         self.assertEqual(response.status_code, 201)
+
+    def test_create_group_membership_no_class_membership(self):
+        self.classroom1_membership.delete()
+        url = reverse("kolibri:core:membership-list")
+        response = self.client.post(
+            url, {"user": self.user.id, "collection": self.lg11.id}, format="json"
+        )
+        self.assertEqual(response.status_code, 400)
+
+
+class DuplicateUsernameTestCase(APITestCase):
+    @classmethod
+    def setUpTestData(cls):
+        cls.facility = FacilityFactory.create()
+        cls.user = FacilityUserFactory.create(facility=cls.facility, username="user")
+        cls.url = reverse("kolibri:core:usernameexists")
+        provision_device()
+
+    def test_check_duplicate_username_with_unique_username(self):
+        response = self.client.get(
+            self.url,
+            {"username": "new_user", "facility": self.facility.id},
+            format="json",
+        )
+        expected = {"username_exists": False}
+        self.assertDictEqual(response.data, expected)
+
+    def test_check_duplicate_username_with_existing_username(self):
+        response = self.client.get(
+            self.url,
+            {"username": self.user.username, "facility": self.facility.id},
+            format="json",
+        )
+        expected = {"username_exists": True}
+        self.assertDictEqual(response.data, expected)
+
+    def test_check_duplicate_username_with_existing_username_other_facility(self):
+        other_facility = FacilityFactory.create()
+        response = self.client.get(
+            self.url,
+            {"username": self.user.username, "facility": other_facility.id},
+            format="json",
+        )
+        expected = {"username_exists": False}
+        self.assertDictEqual(response.data, expected)
