@@ -13,8 +13,6 @@ from sqlalchemy import event
 from sqlalchemy.ext.automap import automap_base
 from sqlalchemy.ext.automap import generate_relationship
 from sqlalchemy.orm import interfaces
-from sqlalchemy.orm import scoped_session
-from sqlalchemy.orm import sessionmaker
 from sqlalchemy.pool import NullPool
 from sqlalchemy.sql import operators
 from sqlalchemy.sql.elements import UnaryExpression
@@ -37,8 +35,6 @@ def set_sqlite_connection_pragma(dbapi_connection, connection_record):
 
 logger = logging.getLogger(__name__)
 
-BASES = {}
-
 
 class ClassNotFoundError(Exception):
     pass
@@ -94,40 +90,27 @@ def get_engine(connection_string):
     Get a SQLAlchemy engine that allows us to connect to a database.
     """
     # Set echo to False, as otherwise we get full SQL Query outputted, which can overwhelm the terminal
-    engine = create_engine(
-        connection_string,
-        echo=False,
-        # Set timeout to 60s, as with most of our content import write operations
+    # Set convert_unicode to True, to properly handle unicode in the DB
+    engine_kwargs = {"echo": False, "convert_unicode": True}
+
+    if connection_string.startswith("sqlite"):
+        # Set timeout to 300s, as with most of our content import write operations
         # it is more important to complete, than to do so quickly.
-        connect_args={"check_same_thread": False, "timeout": 60}
-        if connection_string.startswith("sqlite")
-        else {},
-        poolclass=NullPool,
-        convert_unicode=True,
-    )
+        engine_kwargs["connect_args"] = {"check_same_thread": False, "timeout": 5 * 60}
+    else:
+        engine_kwargs["pool_pre_ping"] = True
+
+    engine = create_engine(connection_string, **engine_kwargs)
     if connection_string == get_default_db_string() and connection_string.startswith(
         "sqlite"
     ):
         event.listen(engine, "connect", set_sqlite_connection_pragma)
         connection = engine.connect()
         connection.execute(START_PRAGMAS)
-        connection.close()
 
     return engine
 
 
-def make_session(connection_string):
-    """
-    Make a session for a particular SQLAlchemy database, this handles opening a connection
-    from the engine, and will automatically start transactions for us.
-    We use the autoflush option to the sessionmaker in order to have explicit control over
-    when we actually commit to the database.
-    """
-    engine = get_engine(connection_string)
-    Session = scoped_session(sessionmaker(bind=engine, autoflush=False))
-    return Session(), engine
-
-
 def get_class(DjangoModel, Base):
     """
     Given a DjangoModel and SQLAlachemy Base mapping that has undergone reflection to have
@@ -178,19 +161,10 @@ def set_all_class_defaults(Base):
             pass
 
 
-__SQLALCHEMY_CLASSES_PATH = (
-    "contentschema",
-    "versions",
-)
+__SQLALCHEMY_CLASSES_PATH = ("contentschema", "versions")
 
 __SQLALCHEMY_CLASSES_MODULE_NAME = "content_schema_{name}"
 
-SQLALCHEMY_CLASSES_PATH_TEMPLATE = os.path.join(
-    os.path.dirname(__file__),
-    "..",
-    *(__SQLALCHEMY_CLASSES_PATH + (__SQLALCHEMY_CLASSES_MODULE_NAME + ".py",))
-)
-
 SQLALCHEMY_CLASSES_MODULE_PATH_TEMPLATE = ".".join(
     tuple(__name__.split(".")[:-2])
     + __SQLALCHEMY_CLASSES_PATH
@@ -213,18 +187,30 @@ def load_metadata(name):
     return module.Base.metadata
 
 
-def prepare_bases():
+class LazyBases(object):
+    _valid_bases = set(CONTENT_DB_SCHEMA_VERSIONS + [CURRENT_SCHEMA_VERSION])
+    _loaded_bases = {}
 
-    for name in CONTENT_DB_SCHEMA_VERSIONS + [CURRENT_SCHEMA_VERSION]:
-        try:
-            metadata = load_metadata(name)
-            BASES[name] = prepare_base(metadata, name=name)
-        except ImportError:
-            logger.error(
-                "Tried to load content schema version {} but valid schema import was not found".format(
-                    name
+    def __getitem__(self, name):
+        if name not in self._valid_bases:
+            raise AttributeError
+        if name not in self._loaded_bases:
+            try:
+                metadata = load_metadata(name)
+                self._loaded_bases[name] = prepare_base(metadata, name=name)
+            except ImportError:
+                logger.error(
+                    "Tried to load content schema version {} but valid schema import was not found".format(
+                        name
+                    )
                 )
-            )
+                self._loaded_bases[name] = None
+        if self._loaded_bases[name] is None:
+            raise AttributeError
+        return self._loaded_bases[name]
+
+
+BASES = LazyBases()
 
 
 def get_model_from_cls(cls):
@@ -283,15 +269,14 @@ def get_default_db_string():
     destination_db = settings.DATABASES.get("default")
     if "sqlite" in destination_db["ENGINE"]:
         return sqlite_connection_string(destination_db["NAME"])
-    else:
-        return "{dialect}://{user}:{password}@{host}{port}/{dbname}".format(
-            dialect=destination_db["ENGINE"].split(".")[-1],
-            user=destination_db["USER"],
-            password=destination_db["PASSWORD"],
-            host=destination_db.get("HOST", "localhost"),
-            port=":" + destination_db.get("PORT") if destination_db.get("PORT") else "",
-            dbname=destination_db["NAME"],
-        )
+    return "{dialect}://{user}:{password}@{host}{port}/{dbname}".format(
+        dialect=destination_db["ENGINE"].split(".")[-1],
+        user=destination_db["USER"],
+        password=destination_db["PASSWORD"],
+        host=destination_db.get("HOST", "localhost"),
+        port=":" + destination_db.get("PORT") if destination_db.get("PORT") else "",
+        dbname=destination_db["NAME"],
+    )
 
 
 class SchemaNotFoundError(Exception):
@@ -303,35 +288,31 @@ class Bridge(object):
         if sqlite_file_path is None:
             # If sqlite_file_path is None, we are referencing the Django default database
             self.connection_string = get_default_db_string()
-            self.schema_version = schema_version or CURRENT_SCHEMA_VERSION
+            schema_version = schema_version or CURRENT_SCHEMA_VERSION
         else:
             # Otherwise, we are accessing an external database.
             self.connection_string = sqlite_connection_string(sqlite_file_path)
-            # If the schema_version is defined, then use the schema_version that was
-            # set.
-            if schema_version is not None:
-                self.schema_version = schema_version
+
+        self.engine = get_engine(self.connection_string)
+        # If the schema_version is defined, then use the schema_version that was
+        # set.
+        if schema_version is not None:
+            self.schema_version = schema_version
+        else:
+            # If not, we are probably looking at an imported content db
+            # So we try each of our historical database schema in order to see
+            # which glass slipper fits! If none do, just turn into a pumpkin.
+            for version in CONTENT_DB_SCHEMA_VERSIONS:
+                self.schema_version = version
+                try:
+                    db_matches_schema(BASES[self.schema_version], self.engine)
+                    break
+                except DBSchemaError as e:
+                    logging.debug(e)
             else:
-                # If not, we are probably looking at an imported content db
-                # So we try each of our historical database schema in order to see
-                # which glass slipper fits! If none do, just turn into a pumpkin.
-                for version in CONTENT_DB_SCHEMA_VERSIONS:
-                    self.schema_version = version
-                    self.session, self.engine = make_session(self.connection_string)
-                    try:
-                        db_matches_schema(BASES[self.schema_version], self.session)
-                        break
-                    except DBSchemaError as e:
-                        logging.debug(e)
-                else:
-                    raise SchemaNotFoundError(
-                        "No matching schema found for this database"
-                    )
+                raise SchemaNotFoundError("No matching schema found for this database")
 
         self.Base = BASES[self.schema_version]
-        # We are using scoped sessions, so should always return the same session
-        # in the same thread
-        self.session, self.engine = make_session(self.connection_string)
 
         if schema_version is not None and sqlite_file_path is not None:
             # In this case we are not using the default database, nor have
@@ -344,7 +325,7 @@ class Bridge(object):
             # change in the schema beyond creating tables.
             self.Base.metadata.create_all(self.engine)
 
-        self.connection = None
+        self._connection = None
 
     def get_class(self, DjangoModel):
         return get_class(DjangoModel, self.Base)
@@ -361,15 +342,20 @@ class Bridge(object):
         return conn.connection
 
     def get_connection(self):
-        if self.connection is None:
-            self.connection = self.engine.connect()
         return self.connection
 
+    @property
+    def connection(self):
+        if self._connection is None:
+            self._connection = self.engine.connect()
+        return self._connection
+
+    def execute(self, query, *args):
+        return self.connection.execute(query, *args)
+
     def end(self):
-        # Clean up session
-        self.session.close()
-        if self.connection:
-            self.connection.close()
+        self.connection.close()
+        self.engine.dispose()
 
 
 def filter_by_uuids(field, ids, validate=True, vendor=None):
