@@ -29,6 +29,10 @@ from kolibri.core.device.utils import allow_guest_access
 from kolibri.core.device.utils import device_provisioned
 from kolibri.core.hooks import LogoutRedirectHook
 from kolibri.core.hooks import RoleBasedRedirectHook
+from kolibri.core.theme_hook import BRAND_COLORS
+from kolibri.core.theme_hook import COLOR_V400
+from kolibri.core.theme_hook import PRIMARY
+from kolibri.core.theme_hook import ThemeHook
 
 
 # Modified from django.views.i18n
@@ -89,8 +93,7 @@ def logout_view(request):
         return HttpResponseRedirect(
             next(obj.url for obj in LogoutRedirectHook.registered_hooks)
         )
-    else:
-        return HttpResponseRedirect(reverse("kolibri:core:redirect_user"))
+    return HttpResponseRedirect(reverse("kolibri:core:redirect_user"))
 
 
 def get_urls_by_role(role):
@@ -101,7 +104,11 @@ def get_urls_by_role(role):
 
 def get_url_by_role(role):
     obj = next(
-        (hook for hook in RoleBasedRedirectHook.registered_hooks if role in hook.roles),
+        (
+            hook
+            for hook in RoleBasedRedirectHook.registered_hooks
+            if role in hook.roles and hook.url
+        ),
         None,
     )
 
@@ -149,7 +156,7 @@ class RootURLRedirectView(View):
             )
             if user_kinds.ADMIN in roles:
                 url = url or get_url_by_role(user_kinds.ADMIN)
-            if user_kinds.COACH in roles:
+            if user_kinds.COACH in roles or user_kinds.ASSIGNABLE_COACH in roles:
                 url = url or get_url_by_role(user_kinds.COACH)
             url = url or get_url_by_role(user_kinds.LEARNER)
         else:
@@ -167,6 +174,16 @@ class RootURLRedirectView(View):
 class UnsupportedBrowserView(TemplateView):
     template_name = "kolibri/unsupported_browser.html"
 
+    def get_context_data(self, **kwargs):
+        context = super(UnsupportedBrowserView, self).get_context_data(**kwargs)
+        context["brand_primary_v400"] = (
+            ThemeHook.get_theme()
+            .get(BRAND_COLORS, {})
+            .get(PRIMARY, {})
+            .get(COLOR_V400, "purple")
+        )
+        return context
+
 
 class StatusCheckView(View):
     def get(self, request):
