@@ -17,16 +17,19 @@ import sys
 
 import pytz
 from django.conf import locale
+from morango.constants import settings as morango_settings
 from six.moves.urllib.parse import urljoin
 from tzlocal import get_localzone
 
 import kolibri
 from kolibri.deployment.default.cache import CACHES
+from kolibri.deployment.default.sqlite_db_names import ADDITIONAL_SQLITE_DATABASES
 from kolibri.plugins.utils.settings import apply_settings
 from kolibri.utils import conf
 from kolibri.utils import i18n
 from kolibri.utils.logger import get_logging_config
 
+
 try:
     isolation_level = None
     import psycopg2  # noqa
@@ -70,6 +73,7 @@ INSTALLED_APPS = [
     "django.contrib.staticfiles",
     "django_filters",
     "kolibri.core.auth.apps.KolibriAuthConfig",
+    "kolibri.core.bookmarks",
     "kolibri.core.content",
     "kolibri.core.logger",
     "kolibri.core.notifications.apps.KolibriNotificationsConfig",
@@ -90,6 +94,7 @@ INSTALLED_APPS = [
 MIDDLEWARE = [
     "kolibri.core.analytics.middleware.cherrypy_access_log_middleware",
     "kolibri.core.device.middleware.ProvisioningErrorHandler",
+    "kolibri.core.device.middleware.DatabaseBusyErrorHandler",
     "django.middleware.cache.UpdateCacheMiddleware",
     "kolibri.core.analytics.middleware.MetricsMiddleware",
     "kolibri.core.auth.middleware.KolibriSessionMiddleware",
@@ -145,13 +150,20 @@ if conf.OPTIONS["Database"]["DATABASE_ENGINE"] == "sqlite":
             ),
             "OPTIONS": {"timeout": 100},
         },
-        "notifications_db": {
-            "ENGINE": "django.db.backends.sqlite3",
-            "NAME": os.path.join(conf.KOLIBRI_HOME, "notifications.sqlite3"),
-            "OPTIONS": {"timeout": 100},
-        },
     }
-    DATABASE_ROUTERS = ("kolibri.core.notifications.models.NotificationsRouter",)
+
+    for additional_db in ADDITIONAL_SQLITE_DATABASES:
+        DATABASES[additional_db] = {
+            "ENGINE": "django.db.backends.sqlite3",
+            "NAME": os.path.join(conf.KOLIBRI_HOME, "{}.sqlite3".format(additional_db)),
+            "OPTIONS": {"timeout": 100},
+        }
+
+    DATABASE_ROUTERS = (
+        "kolibri.core.notifications.models.NotificationsRouter",
+        "kolibri.core.device.models.SyncQueueRouter",
+        "kolibri.core.discovery.models.NetworkLocationRouter",
+    )
 
 elif conf.OPTIONS["Database"]["DATABASE_ENGINE"] == "postgres":
     DATABASES = {
@@ -162,6 +174,7 @@ elif conf.OPTIONS["Database"]["DATABASE_ENGINE"] == "postgres":
             "USER": conf.OPTIONS["Database"]["DATABASE_USER"],
             "HOST": conf.OPTIONS["Database"]["DATABASE_HOST"],
             "PORT": conf.OPTIONS["Database"]["DATABASE_PORT"],
+            "TEST": {"NAME": "test"},
         },
         "default-serializable": {
             "ENGINE": "django.db.backends.postgresql",
@@ -171,6 +184,7 @@ elif conf.OPTIONS["Database"]["DATABASE_ENGINE"] == "postgres":
             "HOST": conf.OPTIONS["Database"]["DATABASE_HOST"],
             "PORT": conf.OPTIONS["Database"]["DATABASE_PORT"],
             "OPTIONS": {"isolation_level": isolation_level},
+            "TEST": {"MIRROR": "default"},
         },
     }
 
@@ -201,6 +215,12 @@ EXTRA_LANG_INFO = {
         "name": "Fulfulde (Cameroon)",
         "name_local": "Fulfulde Mbororoore",
     },
+    "el": {
+        "bidi": False,
+        "code": "el",
+        "name": "Greek",
+        "name_local": "Ελληνικά",
+    },
     "es-419": {
         "bidi": False,
         "code": "es-419",
@@ -225,6 +245,24 @@ EXTRA_LANG_INFO = {
         "name": "Gujarati",
         "name_local": "ગુજરાતી",
     },
+    "ha": {
+        "bidi": False,
+        "code": "ha",
+        "name": "Hausa",
+        "name_local": "Hausa",
+    },
+    "id": {
+        "bidi": False,
+        "code": "id",
+        "name": "Indonesian",
+        "name_local": "Bahasa Indonesia",
+    },
+    "ka": {
+        "bidi": False,
+        "code": "ka",
+        "name": "Georgian",
+        "name_local": "ქართული",
+    },
     "km": {"bidi": False, "code": "km", "name": "Khmer", "name_local": "ភាសាខ្មែរ"},
     "nyn": {
         "bidi": False,
@@ -232,6 +270,12 @@ EXTRA_LANG_INFO = {
         "name": "Chichewa, Chewa, Nyanja",
         "name_local": "Chinyanja",
     },
+    "pt-mz": {
+        "bidi": False,
+        "code": "pt-mz",
+        "name": "Portuguese (Mozambique)",
+        "name_local": "Português (Moçambique)",
+    },
     "zh": {
         "bidi": False,
         "code": "zh-hans",
@@ -251,7 +295,7 @@ LANGUAGE_CODE = (
 
 try:
     TIME_ZONE = get_localzone().zone
-except pytz.UnknownTimeZoneError:
+except (pytz.UnknownTimeZoneError, ValueError):
     # Do not fail at this point because a timezone was not
     # detected.
     TIME_ZONE = pytz.utc.zone
@@ -355,3 +399,29 @@ SESSION_EXPIRE_AT_BROWSER_CLOSE = True
 SESSION_COOKIE_AGE = 1200
 
 apply_settings(sys.modules[__name__])
+
+MORANGO_INSTANCE_INFO = os.environ.get(
+    "MORANGO_INSTANCE_INFO",
+    "kolibri.core.auth.constants.morango_sync:CUSTOM_INSTANCE_INFO",
+)
+# prepend our own Morango Operation to handle custom behaviors during sync
+SYNC_OPERATIONS = ("kolibri.core.auth.sync_operations:KolibriSyncOperations",)
+MORANGO_INITIALIZE_OPERATIONS = (
+    SYNC_OPERATIONS + morango_settings.MORANGO_INITIALIZE_OPERATIONS
+)
+MORANGO_SERIALIZE_OPERATIONS = (
+    SYNC_OPERATIONS + morango_settings.MORANGO_SERIALIZE_OPERATIONS
+)
+MORANGO_QUEUE_OPERATIONS = SYNC_OPERATIONS + morango_settings.MORANGO_QUEUE_OPERATIONS
+MORANGO_TRANSFERRING_OPERATIONS = (
+    SYNC_OPERATIONS + morango_settings.MORANGO_TRANSFERRING_OPERATIONS
+)
+MORANGO_DEQUEUE_OPERATIONS = (
+    SYNC_OPERATIONS + morango_settings.MORANGO_DEQUEUE_OPERATIONS
+)
+MORANGO_DESERIALIZE_OPERATIONS = (
+    SYNC_OPERATIONS + morango_settings.MORANGO_DESERIALIZE_OPERATIONS
+)
+MORANGO_CLEANUP_OPERATIONS = (
+    SYNC_OPERATIONS + morango_settings.MORANGO_CLEANUP_OPERATIONS
+)
